/***************************************************
  This is an example for the BMP085 Barometric Pressure & Temp Sensor

  Designed specifically to work with the Adafruit BMP085 Breakout
  ----> https://www.adafruit.com/products/391

  These displays use I2C to communicate, 2 pins are required to
  interface
  Adafruit invests time and resources providing this open source code,
  please support Adafruit and open-source hardware by purchasing
  products from Adafruit!

  Written by Limor Fried/Ladyada for Adafruit Industries.
  BSD license, all text above must be included in any redistribution
 ****************************************************/
 
 /***************************************************
This is an example sketch for the Adafruit 2.2" SPI display.
This library works with the Adafruit 2.2" TFT Breakout w/SD card
----> http://www.adafruit.com/products/1480
Check out the links above for our tutorials and wiring diagrams
These displays use SPI to communicate, 4 or 5 pins are required to
interface (RST is optional)
Adafruit invests time and resources providing this open source code,
please support Adafruit and open-source hardware by purchasing
products from Adafruit!

Written by Limor Fried/Ladyada for Adafruit Industries.
MIT license, all text above must be included in any redistribution
****************************************************/


//Modified by Steve Spence of http://arduinotronics.blogspot.com

// Connect VIN of the BMP180 sensor to 5.0V //Make sure you have a 5v sensor, otherwise use the 3.3v power pin on the Arduino

// Connect GND to Ground
// Connect SCL to A5 on Arduino Uno (pin 21 on the 2560)
// Connect SDA to A4 on Arduino Uno (pin 20 on the 2560)

#include <Wire.h>
#include <Adafruit_BMP085.h>
#include <dht11.h>

#include "SPI.h"
#include "Adafruit_GFX.h"
#include "Adafruit_ILI9340.h"
#include <MemoryFree.h>

#if defined(__SAM3X8E__)
#undef __FlashStringHelper::F(string_literal)
#define F(string_literal) string_literal
#endif

double Celsius;
double oldCelsius;
double Farenheit;
double oldFarenheit;
double Pascals;
double oldPascal;
double InchesHg;
double oldInchesHg;
double AltitudeMeters;
double oldAltitudeMeters;
double AltitudeFeet;
double oldAltitudeFeet;
double Humidity;
double oldHumidity;
double DewPointCelsius;
double oldDewPointCelsius;

//Celsius to Fahrenheit conversion
double Fahrenheit(double celsius)
{
	return 1.8 * celsius + 32;
}

// dewPoint function NOAA
// reference (1) : http://wahiduddin.net/calc/density_algorithms.htm
// reference (2) : http://www.colorado.edu/geography/weather_station/Geog_site/about.htm
//

double dewPoint(double celsius, double humidity)
{
	// (1) Saturation Vapor Pressure = ESGG(T)
	double RATIO = 373.15 / (273.15 + celsius);
	double RHS = -7.90298 * (RATIO - 1);
	RHS += 5.02808 * log10(RATIO);
	RHS += -1.3816e-7 * (pow(10, (11.344 * (1 - 1/RATIO ))) - 1) ;
	RHS += 8.1328e-3 * (pow(10, (-3.49149 * (RATIO - 1))) - 1) ;
	RHS += log10(1013.246);

        // factor -3 is to adjust units - Vapor Pressure SVP * humidity
	double VP = pow(10, RHS - 3) * humidity;

        // (2) DEWPOINT = F(Vapor Pressure)
	double T = log(VP/0.61078);   // temp var
	return (241.88 * T) / (17.558 - T);
}

// delta max = 0.6544 wrt dewPoint()
// 6.9 x faster than dewPoint()
// reference: http://en.wikipedia.org/wiki/Dew_point


dht11 DHT11;
#define DHT11PIN 2
Adafruit_BMP085 bmp;
#define _sclk 13
#define _miso 12
#define _mosi 11
#define _cs 10
#define _dc 9
#define _rst 8
Adafruit_ILI9340 tft = Adafruit_ILI9340(_cs, _dc, _rst);

void setup() {
  Serial.println("DHT11 TEST PROGRAM ");
  Serial.print("LIBRARY VERSION: ");
  Serial.println(DHT11LIB_VERSION);
  Serial.println();
  Serial.begin(9600);
  while (!Serial);
  tft.begin(); 
  tft.fillScreen(ILI9340_BLACK);
  TextInit();
  
  if (!bmp.begin()) {
Serial.println("Could not find a valid BMP085/180 sensor, check wiring!");
while (1) {}
  }
 
}

void loop() {    
    oldCelsius=Celsius;
    oldPascal=Pascals;
    oldHumidity=Humidity;
    Celsius=bmp.readTemperature();    
    Farenheit=((Celsius*9)/5)+32;    
    Pascals=bmp.readPressure();
    InchesHg=Pascals*0.000295333727;
    AltitudeMeters = bmp.readAltitude();       
    AltitudeFeet = AltitudeMeters *3.28084;
    Humidity = (float)DHT11.humidity, 2;
    DewPointCelsius = dewPoint(Celsius,Humidity);
    
    Serial.println("DTHT11");
    int chk = DHT11.read(DHT11PIN);
    Serial.print("Read sensor: ");
    switch (chk)
    {
    case DHTLIB_OK: 
		Serial.println("OK"); 
		break;
    case DHTLIB_ERROR_CHECKSUM: 
		Serial.println("Checksum error"); 
		break;
    case DHTLIB_ERROR_TIMEOUT: 
		Serial.println("Time out error"); 
		break;
    default: 
    		Serial.println("Unknown error"); 
  		break;
    }
    Serial.print("Humidity [%]: ");
    Serial.println((float)DHT11.humidity, 2);
    Serial.print("Temperature [C]: ");
    Serial.println((float)DHT11.temperature, 2);
    Serial.print("Temperature [F]: ");
    Serial.println(Fahrenheit(DHT11.temperature), 2);
    Serial.print("Dew Point [C]: ");
    Serial.println(dewPoint(DHT11.temperature, DHT11.humidity));
    Serial.println("BMP180");
    Serial.print("Temperature (Celsius):");
    Serial.println(Celsius);
    Serial.print("Temperature (Farenheit):");
    Serial.println(Farenheit);
    Serial.print("Pressure (Pascals):");   
    Serial.println(Pascals);
    Serial.print("Pressure (Inches of Mercury):");  
    Serial.println(InchesHg);
    Serial.print("Altitude = ");
    Serial.print(AltitudeMeters);

  // you can get a more precise measurement of altitude
  // if you know the current sea level pressure which will
  // vary with weather and such. If it is 1015 millibars
  // that is equal to 101500 Pascals.
  //  Serial.print("Real altitude = ");
  //  Serial.print(bmp.readAltitude(101500));
  //  Serial.println(" meters");


  
  Serial.print("freeMemory()=");
  Serial.println(freeMemory());
  Serial.println();
  Serial.println();

  Text();

    delay(5000);
} 

unsigned long TextInit() {
  unsigned long start = micros();
  tft.setCursor(4, 4);
  
  tft.setTextColor(ILI9340_WHITE); tft.setTextSize(2);
  tft.println("Tempreture (C):");  
  tft.setTextColor(ILI9340_GREEN); tft.setTextSize(2);
  tft.println(Celsius);
  tft.setTextColor(ILI9340_WHITE); tft.setTextSize(2);
  tft.println("Tempreture (F):");  
  tft.setTextColor(ILI9340_GREEN); tft.setTextSize(2);
  tft.println(Farenheit);
  
  tft.setTextColor(ILI9340_WHITE); tft.setTextSize(2);
  tft.println("Pressure (Pa):");  
  tft.setTextColor(ILI9340_GREEN); tft.setTextSize(2);
  tft.println(Pascals);
  tft.setTextColor(ILI9340_WHITE); tft.setTextSize(2);
  tft.println("Pressure (Hg):");  
  tft.setTextColor(ILI9340_GREEN); tft.setTextSize(2);
  tft.println(InchesHg);
  
  tft.setTextColor(ILI9340_WHITE); tft.setTextSize(2);
  tft.println("Humidity (%):");  
  tft.setTextColor(ILI9340_GREEN); tft.setTextSize(2);
  tft.println(Humidity);
  
  tft.setTextColor(ILI9340_WHITE); tft.setTextSize(2);
  tft.println("Altitude (m):");  
  tft.setTextColor(ILI9340_GREEN); tft.setTextSize(2);
  tft.println(AltitudeMeters);
   
  tft.setTextColor(ILI9340_WHITE); tft.setTextSize(2);
  tft.println("Dew Point (C):");  
  tft.setTextColor(ILI9340_GREEN); tft.setTextSize(2);
  tft.println(DewPointCelsius);
  
  return micros() - start;
}

unsigned long Text() {
  unsigned long start = micros()
  ;
  if (oldCelsius != Celsius){
    tft.setCursor(4, 4);
    tft.setTextColor(ILI9340_GREEN); tft.setTextSize(2); 
    tft.nextLine();
    tft.println(Celsius);       
    tft.nextLine(); 
    tft.println(Farenheit);     
    tft.nextLine();
  }
  
    if (oldPascal != Pascals){
    tft.setCursor(4, 4);  
    tft.setTextColor(ILI9340_GREEN); tft.setTextSize(2);      
    tft.nextLine();       
    tft.nextLine();       
    tft.nextLine();       
    tft.nextLine();
    tft.nextLine(); 
    tft.println(Pascals);     
    tft.nextLine();
    tft.println(InchesHg); 
  }

    if (oldHumidity != Humidity){
    tft.setCursor(4, 4);  
    tft.setTextColor(ILI9340_GREEN); tft.setTextSize(2);   
    tft.nextLine();       
    tft.nextLine();       
    tft.nextLine();       
    tft.nextLine();
    tft.nextLine(); 
    tft.nextLine();      
    tft.nextLine();
    tft.nextLine(); 
    tft.nextLine(); 
    tft.println(Humidity); 
    }
    
if (oldAltitudeMeters != AltitudeMeters){
    tft.setCursor(4, 4);  
    tft.setTextColor(ILI9340_GREEN); tft.setTextSize(2);   
    tft.nextLine();       
    tft.nextLine();       
    tft.nextLine();       
    tft.nextLine();
    tft.nextLine(); 
    tft.nextLine();      
    tft.nextLine();
    tft.nextLine(); 
    tft.nextLine();
    tft.nextLine(); 
    tft.nextLine(); 
    tft.println(AltitudeMeters); 
    }
    
if (oldDewPointCelsius != DewPointCelsius){
    tft.setCursor(4, 4);  
    tft.setTextColor(ILI9340_GREEN); tft.setTextSize(2);   
    tft.nextLine();       
    tft.nextLine();       
    tft.nextLine();       
    tft.nextLine();
    tft.nextLine(); 
    tft.nextLine();      
    tft.nextLine();
    tft.nextLine(); 
    tft.nextLine();
    tft.nextLine(); 
    tft.nextLine(); 
    tft.nextLine(); 
    tft.nextLine(); 
    tft.println(DewPointCelsius); 
    }

  return micros() - start;
}
